#ifndef SPASERVER_H
#define SPASERVER_H

#include "FromSuperBrain.h"
#include "FromRedis.h"
#include "FromWebAPI.h"

class FuncBase;
class FuncThreadInfo;


/**
 * 安播辅助提示系统服务类
 *      写入EQM数据库的报警逻辑
 * 
 */
class SPAServer
{
public:
    SPAServer();
    ~SPAServer();

    /* 启动服务 */
    void startServer();

private:
    /*--------------------------------------------------------------------------------
     * 从基础平台获取算法信息和设备信息功能线程和所需要的函数，这里更新维护EQM数据库的tAction、
     * tCameraInfo、tActionCamera等表格
     *-------------------------------------------------------------------------------*/
    /* 从基础平台获取算法信息和设备信息的线程函数 */
    void threadFromSuperBrain();
    /* 处理算法信息，返回值为true，说明有改变，需要重新读取 */
    bool processAlgorithmInfo(std::vector<AlgorithmInfo> vecNewAlgInfo);
    /* 处理设备信息，传入新获取到的值，返回true需要更新本地数据缓存 */
    bool processDeviceInfo(std::vector<DeviceInfo> vecNewDevInfo);
    /* 对比算法信息现有的数据和新获取到的数据，取出要删除和添加的数据 */
    void compareAlgorithmInfo(const std::vector<AlgorithmInfo>& vecNewInfo, std::vector<AlgorithmInfo>& vecAlgUpdate, std::vector<AlgorithmInfo>& vecAlgDelete);
    /* 对比设备和算法关联表是否需要更新 */
    void compareDeviceAlgorithmInfo(const std::vector<DeviceInfo>& vecNewInfo, std::vector<DeviceInfo>& vecDevUpdate);

    /* 从Redis获取数据线程函数，这个是摄像机线程（注意，这个函数未被使用，不从这里获取Redis数据） */
    void threadFromRedis(const CameraThreadInfo& info);

    /*--------------------------------------------------------------------------------
     * 分派任务的线程，这里是根据房间和摄像机的关联信息，给任务创建不同的功能线程
     *-------------------------------------------------------------------------------*/
    /* 分派任务的线程 */
    void threadDistribution();
    /* 创建任务实例的函数 */
    FuncBase* createFuncInstance(FuncThreadInfo& info);
    /* 清理没有在运行的线程实例 */
    void clearNoneFuncThreadInfo();
    
    /* 区域人员检测(人员计数)，检测这个区域内的人数，不能少于多少人，不能多余多少人 */
    void threadActRegionalPersonnelDetection(FuncThreadInfo* RFAInfo);


private:
    std::shared_ptr<spdlog::logger> m_logger = nullptr;

    bool m_threadRunning = true;                        /* 线程正在运行 */

    FromSuperBrain m_fromSuperBrain;
    FromWebAPI m_fromWebAPI;

    /* 算法信息，这个就是tAction在内存中的数据，方便后续对比，程序启动的时候会先获取一份 */
    std::vector<AlgorithmInfo> m_vecEqmAlgInfo;
    /* 设备信息，这个是tActionCamer的信息 */
    std::vector<DeviceInfo> m_vecEqmDevInfo;
    /* 算法信息列表，每个算法所在的频率、房间、摄像机 */
    std::mutex m_mutexActionInfo;
    ListActionInfo m_listActionInfo;
    /* 设备和算法关联信息，这里存储着已经删除的设备对应的算法信息，将在这一轮循环中删除 */
    std::list<int> m_listDevIDDelete;

    /* 线程功能类实例列表 */
    std::list<FuncBase*> m_listFuncBase;

};

#endif /* SPASERVER_H */
